'''
CF-1.2 changes based on CF-1.1 checker code

implementation of requirements and recommendations from
http://cf-pcmdi.llnl.gov/conformance/requirements-and-recommendations/1.2/ and
http://cf-pcmdi.llnl.gov/documents/cf-conventions/1.2/cf-conventions.html

this code is an extension of the CF-1.0 checker base code in the cf10 module
and the modifications by the CF-1.1 checker code in the cf11 module
'''
from CFchecker import cf11

class CF12Tests(cf11.CF11Tests):
    '''
    changes compared to 1.1: more grid_mappings added
    '''
    cf_convention = 'CF-1.2'

    @classmethod
    def _setup_grid_mappings(cls):
        '''
        extend dict of grid mapping names with their allowed attributes
        '''
        # get mappings from base class
        mappings = cf11.CF11Tests._setup_grid_mappings()

        #extend mappings
        # there is an inconsistency in the CF-1.2 standard about latitude_longitude mapping attributes:
        # some are listed in an example, but nowhere else, we are using the ones from the example here
        # as a best guess
        mappings['latitude_longitude'] = ['longitude_of_prime_meridian', 'semi_major_axis', 'inverse_flattening']
        mappings['vertical_perspective'] = ['latitude_of_projection_origin', 'longitude_of_projection_origin',
                                            'perspective_point_height', 'false_easting', 'false_northing']

        return mappings

    @classmethod
    def _setup_grid_mapping_constraints(cls):
        '''
        extend constraints dict for grid mapping attributes
        '''
        eps = cls.epsilon
        inf = float('inf')

        # get constraints from base class
        constraints = cf11.CF11Tests._setup_grid_mapping_constraints()

        # extend constraints
        # in addition to range and size, a type constraint has been introduced by CF-1.2 as well
        # ADDITION: the lower limit is not given in the standard, but a radius should be > 0
        constraints['earth_radius'] =                   ['N', (0+eps, inf), (1,1)]
        # ADDITION: lower limit should not be negative as f=(a-b)/a cannot be negative for a>=b and a,b>0
        constraints['inverse_flattening'] =             ['N', (0, inf), (1,1)]
        constraints['longitude_of_prime_meridian'] =    ['N', (-180.0, 180.0-eps), (1,1)]
        constraints['perspective_point_height'] =       ['N', (-inf, inf), (1,1)]
        # ADDITION: lower limit should be > 0 for any ellipsoid
        constraints['semi_major_axis'] =                ['N', (0+eps, inf), (1,1)]
        constraints['perspective_point_height'] =       ['N', (-inf, inf), (1,1)]
        # ADDITION: lower limit should be > 0 for any ellipsoid
        constraints['semi_minor_axis'] =                ['N', (0+eps, inf), (1,1)]

        return constraints

